@@ -11,11 +11,11 @@ use core::ops::{
     BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, Bound, Not, Range, RangeFrom,
     RangeTo, Sub, SubAssign,
 };
-use ref_cast::RefCast;
+use ref_cast::{ref_cast_custom, RefCastCustom};
 use smallvec::{Array, SmallVec};
 use std::borrow::Borrow;
 use std::num::*;
-use std::ops::{Deref, RangeBounds};
+use std::ops::{Deref, RangeBounds, RangeFull};
 #[cfg(feature = "serde")]
 use {
     core::{fmt, marker::PhantomData},
@@ -239,13 +239,18 @@ impl<'a, T> Iterator for Iter<'a, T> {
 }
 
 /// A reference to a range set
-#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, RefCast)]
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, RefCastCustom)]
 #[repr(transparent)]
 pub struct RangeSetRef<T>([T]);
 
 impl<T> RangeSetRef<T> {
+    /// Create a new range set reference for an empty range set
+    pub const fn empty() -> &'static Self {
+        RangeSetRef::new_unchecked_impl(&[])
+    }
+
     /// Create a new range set reference for a single value
-    pub fn single(value: &T) -> &Self {
+    pub const fn single(value: &T) -> &Self {
         RangeSetRef::new_unchecked_impl(std::slice::from_ref(value))
     }
 
@@ -290,17 +295,15 @@ impl<T> RangeSetRef<T> {
     /// Create a new range set reference without checking that the boundaries are
     /// strictly sorted.
     #[cfg(feature = "new_unchecked")]
-    pub fn new_unchecked(boundaries: &[T]) -> &Self {
+    pub const fn new_unchecked(boundaries: &[T]) -> &Self {
         Self::new_unchecked_impl(boundaries)
     }
 
-    #[inline]
-    fn new_unchecked_impl(boundaries: &[T]) -> &Self {
-        Self::ref_cast(boundaries)
-    }
+    #[ref_cast_custom]
+    const fn new_unchecked_impl(boundaries: &[T]) -> &Self;
 
     /// The boundaries of the range set, guaranteed to be strictly sorted
-    pub fn boundaries(&self) -> &[T] {
+    pub const fn boundaries(&self) -> &[T] {
         &self.0
     }
 
@@ -316,7 +319,7 @@ impl<T> RangeSetRef<T> {
     }
 
     /// true if the range set is empty
-    pub fn is_empty(&self) -> bool {
+    pub const fn is_empty(&self) -> bool {
         self.boundaries().is_empty()
     }
 
@@ -464,6 +467,7 @@ pub trait RangeSetEntry: Ord {
 macro_rules! entry_instance {
     ($t:tt) => {
         impl RangeSetEntry for $t {
+
             fn min_value() -> Self {
                 $t::MIN
             }
@@ -482,6 +486,7 @@ macro_rules! entry_instance {
 macro_rules! non_zero_u_entry_instance {
     ($t:tt) => {
         impl RangeSetEntry for $t {
+
             fn min_value() -> Self {
                 $t::new(1).unwrap()
             }
@@ -686,12 +691,39 @@ impl<T: RangeSetEntry, A: Array<Item = T>> From<RangeTo<T>> for RangeSet<A> {
     }
 }
 
+impl<T: RangeSetEntry, A: Array<Item = T>> From<RangeFull> for RangeSet<A> {
+    fn from(_: RangeFull) -> Self {
+        Self::all()
+    }
+}
+
 /// compute the intersection of this range set with another, producing a new range set
 ///
 /// &forall; t &isin; T, r(t) = a(t) & b(t)
-impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitAnd for &RangeSet<A> {
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitAnd<RangeSet<A>> for RangeSet<A> {
     type Output = RangeSet<A>;
-    fn bitand(self, that: Self) -> Self::Output {
+    fn bitand(self, that: RangeSet<A>) -> Self::Output {
+        self.intersection(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitAnd<&RangeSet<A>> for RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitand(self, that: &RangeSet<A>) -> Self::Output {
+        self.intersection(that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitAnd<RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitand(self, that: RangeSet<A>) -> Self::Output {
+        self.intersection(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitAnd<&RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitand(self, that: &RangeSet<A>) -> Self::Output {
         self.intersection(that)
     }
 }
@@ -705,9 +737,30 @@ impl<T: Ord, A: Array<Item = T>> BitAndAssign for RangeSet<A> {
 /// compute the union of this range set with another, producing a new range set
 ///
 /// &forall; t &isin; T, r(t) = a(t) | b(t)
-impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitOr for &RangeSet<A> {
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitOr<RangeSet<A>> for RangeSet<A> {
     type Output = RangeSet<A>;
-    fn bitor(self, that: Self) -> Self::Output {
+    fn bitor(self, that: RangeSet<A>) -> Self::Output {
+        self.union(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitOr<&RangeSet<A>> for RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitor(self, that: &RangeSet<A>) -> Self::Output {
+        self.union(that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitOr<RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitor(self, that: RangeSet<A>) -> Self::Output {
+        self.union(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitOr<&RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitor(self, that: &RangeSet<A>) -> Self::Output {
         self.union(that)
     }
 }
@@ -721,9 +774,30 @@ impl<T: Ord, A: Array<Item = T>> BitOrAssign for RangeSet<A> {
 /// compute the exclusive or of this range set with another, producing a new range set
 ///
 /// &forall; t &isin; T, r(t) = a(t) ^ b(t)
-impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitXor for &RangeSet<A> {
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitXor<RangeSet<A>> for RangeSet<A> {
     type Output = RangeSet<A>;
-    fn bitxor(self, that: Self) -> Self::Output {
+    fn bitxor(self, that: RangeSet<A>) -> Self::Output {
+        self.symmetric_difference(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitXor<&RangeSet<A>> for RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitxor(self, that: &RangeSet<A>) -> Self::Output {
+        self.symmetric_difference(that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitXor<RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitxor(self, that: RangeSet<A>) -> Self::Output {
+        self.symmetric_difference(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> BitXor<&RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn bitxor(self, that: &RangeSet<A>) -> Self::Output {
         self.symmetric_difference(that)
     }
 }
@@ -737,9 +811,30 @@ impl<T: RangeSetEntry, A: Array<Item = T>> BitXorAssign for RangeSet<A> {
 /// compute the difference of this range set with another, producing a new range set
 ///
 /// &forall; t &isin; T, r(t) = a(t) & !b(t)
-impl<T: RangeSetEntry + Clone, A: Array<Item = T>> Sub for &RangeSet<A> {
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> Sub<RangeSet<A>> for RangeSet<A> {
     type Output = RangeSet<A>;
-    fn sub(self, that: Self) -> Self::Output {
+    fn sub(self, that: RangeSet<A>) -> Self::Output {
+        self.difference(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> Sub<&RangeSet<A>> for RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn sub(self, that: &RangeSet<A>) -> Self::Output {
+        self.difference(that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> Sub<RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn sub(self, that: RangeSet<A>) -> Self::Output {
+        self.difference(&that)
+    }
+}
+
+impl<T: RangeSetEntry + Clone, A: Array<Item = T>> Sub<&RangeSet<A>> for &RangeSet<A> {
+    type Output = RangeSet<A>;
+    fn sub(self, that: &RangeSet<A>) -> Self::Output {
         self.difference(that)
     }
 }
@@ -1115,6 +1210,10 @@ fn is_strictly_sorted<T: Ord>(ranges: &[T]) -> bool {
 /// `left`, `right` at position `at`, so that
 ///   contains(left, x) == contains(ranges, x) for x < at
 ///   contains(right, x) == contains(ranges, x) for x >= at
+///
+/// The returned slices are guaranteed to be minimal. The left slice will contain
+/// no boundaries after the split point, and the right slice will contain at most
+/// one boundary before the split point.
 #[inline]
 fn split<T: Ord>(ranges: &[T], at: T) -> (&[T], &[T]) {
     let l = ranges.len();
@@ -1134,7 +1233,7 @@ fn split<T: Ord>(ranges: &[T], at: T) -> (&[T], &[T]) {
             // since i is an odd value, it indicates going to false at the
             // split point, and we don't need to have it in right.
             let sp = i.saturating_add(1).min(l);
-            (&ranges[..sp], &ranges[sp..])
+            (&ranges[..i], &ranges[sp..])
         }
         Err(i) => {
             // left will be an odd size, so we add one if possible
@@ -1142,10 +1241,7 @@ fn split<T: Ord>(ranges: &[T], at: T) -> (&[T], &[T]) {
             // i is an odd value, so right is true at the split point, and
             // we need to add one value before the split point to right.
             // hence the saturating_sub(1).
-            (
-                &ranges[..i.saturating_add(1).min(l)],
-                &ranges[i.saturating_sub(1)..],
-            )
+            (&ranges[..i], &ranges[i.saturating_sub(1)..])
         }
     }
 }
@@ -1217,7 +1313,7 @@ mod util_tests {
                 let nr = right.iter().filter(|x| x < &&at).count();
                 prop_assert!(nr <= 1, "there must be at most one boundary before the split point");
                 let nl = left.iter().filter(|x| x >= &&at).count();
-                prop_assert!(nl <= 1, "there must be at most one boundary after the split point");
+                prop_assert!(nl == 0, "there must be no boundaries after the split point");
             }
         }
     }
@@ -1227,13 +1323,13 @@ mod util_tests {
         #[allow(clippy::type_complexity)]
         let cases: Vec<(&[u64], u64, (&[u64], &[u64]))> = vec![
             (&[0, 2], 0, (&[], &[0, 2])),
-            (&[0, 2], 2, (&[0, 2], &[])),
-            (&[0, 2, 4], 2, (&[0, 2], &[4])),
+            (&[0, 2], 2, (&[0], &[])),
+            (&[0, 2, 4], 2, (&[0], &[4])),
             (&[0, 2, 4], 4, (&[0, 2], &[4])),
-            (&[0, 2, 4, 8], 2, (&[0, 2], &[4, 8])),
+            (&[0, 2, 4, 8], 2, (&[0], &[4, 8])),
             (&[0, 2, 4, 8], 4, (&[0, 2], &[4, 8])),
             (&[0, 2, 4, 8], 3, (&[0, 2], &[4, 8])),
-            (&[0, 2, 4, 8], 6, (&[0, 2, 4, 8], &[4, 8])),
+            (&[0, 2, 4, 8], 6, (&[0, 2, 4], &[4, 8])),
         ];
         for (ranges, pos, (left, right)) in cases {
             assert_eq!(split(ranges, pos), (left, right));
@@ -1341,8 +1437,8 @@ mod tests {
 
         println!("{:?} {:?} {:?} {:?}", x, y, z, r);
 
-        let r2: Test = x.bitand(&y);
-        let r3: Test = x.bitxor(&y);
+        let r2: Test = &x & &y;
+        let r3: Test = &x | &y;
         let r4 = y.is_disjoint(&z);
         let r5 = (&y).bitand(&z);
 
